unit fmod_devselect;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, fmod, fmodtypes, INIFiles;

type
  TConfigForm = class(TForm)
    GroupBox1: TGroupBox;
    SoundServices: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    OutputDevices: TListBox;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure SoundServicesClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  ConfigForm: TConfigForm;

implementation

{$R *.DFM}
const
  {$IFDEF WIN32}
  OutputTypes: array [0..3] of String =
    ('Direct Sound', 'Windows Multimedia System', 'Aureal 3D', 'NoSound');
  SoundOutputTypes : Array[0..High(OutputTypes)] of TFSoundOutputTypes =
    (FSOUND_OUTPUT_DSOUND, FSOUND_OUTPUT_WINMM, FSOUND_OUTPUT_A3D, FSOUND_OUTPUT_NOSOUND);
  {$ENDIF}
  {$IFDEF LINUX}
  OutputTypes: array [0..3] of String =
    ('Open Sound System', 'Enlightment Sound Daemon', 'Alsa', 'NoSound');
  SoundOutputTypes : Array[0..High(OutputTypes)] of TFSoundOutputTypes =
    (FSOUND_OUTPUT_OSS, FSOUND_OUTPUT_ESD, FSOUND_OUTPUT_ALSA, FSOUND_OUTPUT_NOSOUND);
  {$ENDIF}

procedure TConfigForm.FormCreate(Sender: TObject);
var
  i : Integer;
  Reg : TINIFile;
begin
  //Sound Services anzeigen
  SoundServices.Clear;
  for i := 0 to High(OutputTypes) do
  begin
    SoundServices.Items.Add(OutPutTypes[i]);
  end;

  //Vorauswahl
  SoundServices.ItemIndex := 1; //Windows Multimedia System
  try
    Reg := TINIFile.Create(ExtractFileDir(ParamStr(0))+'\sound.ini');
    SoundServices.ItemIndex := Reg.ReadInteger('FMOD', 'SoundDriver', 1);
    SoundServicesClick(Self); //Soundtreiber anzeigen
    OutputDevices.ItemIndex := Reg.ReadInteger('FMOD', 'SoundDevice', 0);
  finally
    Reg.Free;
  end;
end;

procedure TConfigForm.SoundServicesClick(Sender: TObject);
var
  i : Integer;
begin
  OutputDevices.Clear;
  //Je nach Service entsprechende Treiber anzeigen
  if (SoundServices.ItemIndex >= 0) and (SoundServices.ItemIndex <= High(OutputTypes)) then
  begin
    FSOUND_SetOutput(SoundOutputTypes[SoundServices.ItemIndex]);
    for i := 0 to FSOUND_GetNumDrivers - 1 do
      OutputDevices.Items.Add(FSOUND_GetDriverName(i));
    OutputDevices.ItemIndex := 0;
    FSOUND_Close;
  end;
end;

procedure TConfigForm.Button1Click(Sender: TObject);
var
  Reg : TINIFile;
begin
  try
    Reg := TINIFile.Create(ExtractFileDir(ParamStr(0))+'\sound.ini');
    Reg.WriteInteger('FMOD', 'SoundDriver', SoundServices.ItemIndex);
    Reg.WriteInteger('FMOD', 'SoundDevice', OutputDevices.ItemIndex);
  finally
    Reg.Free;
  end;
end;

end.
