unit fmod_synthutils;

interface

function OpenFMOD : Boolean;
procedure CloseFMOD;

implementation

uses
  SysUtils, engine_rtf, fmod, fmodtypes, INIFiles;

const
  OutputTypes: array [0..3] of String =
    ('Direct Sound', 'Windows Multimedia System', 'Aureal 3D', 'NoSound');
  SoundOutputTypes : Array[0..High(OutputTypes)] of TFSoundOutputTypes =
    (FSOUND_OUTPUT_DSOUND, FSOUND_OUTPUT_WINMM, FSOUND_OUTPUT_A3D, FSOUND_OUTPUT_NOSOUND);

function OpenFMOD : Boolean;
var
  Serv, Dev : Integer;
  Reg : TINIFile;
begin
  Result := False;
  try
    if FMOD_VERSION > FSOUND_GetVersion then
      LogFile.AddWarning('sutils->OpenFMOD', Format(
        'Benutze ltere fmod Bibliothek als %f. Installieren Sie eine '+
        'neuere Version, um Probleme zu vermeiden!', [FMOD_VERSION]))
    else if FMOD_VERSION < FSOUND_GetVersion then
      LogFile.AddWarning('sutils->OpenFMOD', Format(
        'Benutze neuere fmod Bibliothek als %f' +
        '. Diese Version wurde mit diesem Programm nicht getestet!', [FMOD_VERSION]));


    Reg := TINIFile.Create(ExtractFileDir(ParamStr(0))+'\sound.ini');
    Serv := Reg.ReadInteger('FMOD', 'SoundDriver', 1);
    if (Serv < 0) or (Serv > High(OutPutTypes)) then
      Serv := 0;
    if FSOUND_SetOutput(SoundOutputTypes[Serv]) then
    begin
      Dev := Reg.ReadInteger('FMOD', 'SoundDevice', 0);
      if FSOUND_SetDriver(Dev) then
      begin
        Result := true;
        LogFile.AddSuccess('sutils->OpenFMOD', 'Soundausgabe erfolgreich aktiviert');
        LogFile.AddMessage('sutils->OpenFMOD', 'Sound Ausgabedienst: [B] ' + OutputTypes[Serv]);
        LogFile.AddMessage('sutils->OpenFMOD', 'Sound Ausgabegeraet: [B] ' + FSOUND_GetDriverName(Dev));
      end
      else
        LogFile.AddError('sutils->OpenFMOD', 'Ausgabegert konnte nicht gewhlt werden! => mit fmodconf konfigurieren',0);
    end
    else
      LogFile.AddError('sutils->OpenFMOD', 'Ausgabedienst konnte nicht gewhlt werden! => mit fmodconf konfigurieren',0);
  finally
    Reg.Free;
  end;
end;

procedure CloseFMOD;
begin
  LogFile.AddMessage('sutils->CloseFMOD', 'Soundausgabe deaktiviert');
  FSOUND_Close;
end;

end.
