unit spectrum_analyzer;

interface

uses
  Windows, Classes, Controls, SysUtils, Graphics, FMOD;

type
  TSpectrum = class(TGraphicControl)
              private
                FBackColor : TColor;
                FSpecEnabled   : Boolean;

                Gradient : TBitMap;
                BackBuffer : TBitMap;
                procedure CreateGradient;
              protected
                procedure Paint; override;
                procedure Resize; override;

                procedure SetSpecEnabled(Value : Boolean);
              public
                property BackgroundColor : TColor read FBackColor write FBackColor;
                property SpectrumEnabled : Boolean read FSpecEnabled write SetSpecEnabled;

                constructor Create(AOwner: TComponent); override;
                destructor Destroy; override;
              end;

implementation

{ TSpectrum }

constructor TSpectrum.Create(AOwner: TComponent);
begin
  inherited;
  Width := 256;
  Height := 96;
  FBackColor := clBlack;
  FSpecEnabled := False;
  CreateGradient;
  BackBuffer := TBitMap.Create;
end;

procedure TSpectrum.CreateGradient;
var
  i : Integer;
  m : Double;
  r,g,b : Byte;
begin
  if Height <= 0 then Exit;

  if Gradient <> nil then
  begin
    Gradient.Free;
    Gradient := nil
  end;

  Gradient := TBitMap.Create;
  Gradient.Width := 4;
  Gradient.Height := Height;
  m := 256.0 / Height;
  for i := 0 to Height - 1 do
  begin
    b := Trunc(i * m);
    r := Trunc(255 - i * m);
    g := Trunc(i * m);
    //Gradient.Canvas.Pixels[0,i] := RGB(r, g, b);
    Gradient.Canvas.Pen.Color := RGB(r, g, b);
    Gradient.Canvas.MoveTo(0, i);
    Gradient.Canvas.LineTo(4, i);
  end;
end;

destructor TSpectrum.Destroy;
begin
  Enabled := False;
  Gradient.Free;
  Gradient := nil;
  BackBuffer.Free;
  BackBuffer := nil;
  inherited;
end;

procedure TSpectrum.Paint;
type
  TFMODSpectrum = packed Array[0..512] of Single;
  PFMODSpectrum = ^TFMODSpectrum;
var
  Spectrum : Array[0..63] of Single;
  FMODSpectrum : PFMODSpectrum;
  i, k : Integer;
  yo, yu : Integer;
begin
  if Assigned(BackBuffer) then
    with BackBuffer.Canvas do
    begin
      Pen.Color := FBackColor;
      Brush.Color := FBackColor;
      Rectangle(0,0, Width -1, Height -1);

      if FSpecEnabled then
      begin
        FMODSpectrum := PFMODSpectrum(FSOUND_DSP_GetSpectrum);
        for i := 0 to 63 do
        begin
          Spectrum[i] := FMODSpectrum[i*8];
          for k := 1 to 7 do
          begin
            if FMODSpectrum[i*8+k] > Spectrum[i] then
              Spectrum[i] := FMODSpectrum[i*8+k]
          end;
          //Spectrum[i] := Spectrum[i] / 8;
          //if Spectrum[i] < 0.02 then Spectrum[i] := 0.0;
        end;

        for i := 0 to 63 do
        begin
          if Spectrum[i] > 0 then
          begin
            yu := Height - 1;
            yo := Trunc(Height - 1 - Spectrum[i] * (Height - 1));
            BitBlt(Handle, i*4, yo, (i+1)*4, yu - yo, Gradient.Canvas.Handle, 0, yo, SRCCOPY);
          end;
        end;
      end;
      //BitBlt(Canvas.Handle, 0, 0, Width, Height, BackBuffer.Handle, 0, 0, SRCCOPY);
      Canvas.Draw(0,0, BackBuffer);
    end;
  //inherited;
end;

procedure TSpectrum.Resize;
begin
  inherited;
  CreateGradient;
  if Assigned(BackBuffer) then
  begin
    BackBuffer.Free;
    BackBuffer := nil;
    BackBuffer := TBitMap.Create;
    BackBuffer.Width := Width;
    BackBuffer.Height := Height;
  end;
end;

procedure TSpectrum.SetSpecEnabled(Value: Boolean);
begin
  FSOUND_DSP_SetActive(FSOUND_DSP_GetFFTUnit, Value);
  FSpecEnabled := Value;
end;

end.
