unit synth1_sinus;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, fmod_synthutils, engine_rtf, ComCtrls;

type
  TSinGenForm = class(TForm)
    FreqBar: TScrollBar;
    FreqLabel: TLabel;
    CPUUsageLabel: TLabel;
    CPUUsage: TProgressBar;
    procedure FreqBarChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    SoundActive : Boolean;
  public
    { Public-Deklarationen }
  end;

var
  SinGenForm: TSinGenForm;

implementation

{$R *.DFM}

uses
  fmod, fmodtypes, fmoderrors;

const
  SamplingRate = 44100;

var
  Stream: PFSoundStream;
  Channel : Integer;
  Pos : Single;
  LastFreq, NewFreq : Integer;
  CPUUpdate : Integer;

function StreamCallback(Stream: PFSoundStream; Buff: Pointer; Len, Param: Integer): ByteBool; stdcall;
var
  Count: Integer;
  Buffer: PChar;
  CPUUsage : Single;
begin
  NewFreq := SinGenForm.FreqBar.Position;
  Buffer := PChar(Buff);
  Count := 0;

  while Count < (Len shr 2) do		{ >>2 = 16bit stereo (4 bytes per sample) }
  begin
    //LastFreq := NewFreq;
    if Pos >= SamplingRate/LastFreq then //2*Pi/SR*f*p >= 2*Pi
    begin
      Pos := Pos - SamplingRate/(LastFreq);
      if (LastFreq <> NewFreq) then
      begin
        LastFreq := NewFreq;
        Pos := 0.0;
      end;
    end;

    PSmallInt(Buffer)^ := Trunc(Sin(2*Pi/SamplingRate*LastFreq*Pos) * (32767.0/2.0));    { Left channel }
    Inc(Buffer, 2);
    PSmallInt(Buffer)^ := Trunc(Sin(2*Pi/SamplingRate*LastFreq*Pos) * (32767.0/2.0));   { Right channel }
    Inc(Buffer, 2);

    Inc(Count);
    Pos := Pos + 1.0;
  end;

  Inc(CPUUpdate);
  if CPUUpdate > 10 then
  begin
    CPUUsage := FSOUND_GetCPUUsage;
    SinGenForm.CPUUsage.Position := Round(CPUUsage*10);
    SinGenForm.CPUUsageLabel.Caption := Format('CPU-Last: %f %%', [CPUUsage]);
    CPUUpdate := 0;
  end;

  Result := True;
end;


procedure TSinGenForm.FreqBarChange(Sender: TObject);
var
  Freq : Integer;
begin
  Freq := FreqBar.Position;
  if Freq < 1000 then
    FreqLabel.Caption := Format('%d Hz Lambda = %f m', [Freq, 331/Freq])
  else
    FreqLabel.Caption := Format('%f kHz Lambda = %f mm', [Freq/1000, 331/Freq*1000])
end;

procedure TSinGenForm.FormCreate(Sender: TObject);
begin
  LastFreq := FreqBar.Position;
  Pos := 0;
  CPUUpdate := 10;
  FreqBarChange(Self);

  SoundActive := OpenFMOD;
  if SoundActive then
  begin
    if not FSOUND_Init(SamplingRate, 16, 0) then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der Initialisierung von FMOD ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;

    Stream := FSOUND_Stream_Create(StreamCallback, 2500 * 2 * 2, FSOUND_NORMAL or FSOUND_16BITS or FSOUND_STEREO, SamplingRate, 12345);
    if Stream = nil then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der des Sound-Streams ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;
    Channel := FSOUND_Stream_Play(FSOUND_FREE, Stream);
  end
  else
    LogFile.AddWarning('->FormCreate', 'Soundausgabe nicht mglich!');
end;

procedure TSinGenForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if SoundActive then
    FSOUND_Stream_Close(Stream);
  CloseFMOD;
end;

end.
