unit synth2_schwebung;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, fmod_synthutils, engine_rtf, ComCtrls;

type
  TSchwebGenForm = class(TForm)
    PercentBar: TScrollBar;
    PercentLabel: TLabel;
    CPUUsageLabel: TLabel;
    CPUUsage: TProgressBar;
    procedure PercentBarChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    SoundActive : Boolean;
  public
    { Public-Deklarationen }
  end;

var
  SchwebGenForm: TSchwebGenForm;

implementation

{$R *.DFM}

uses
  fmod, fmodtypes, fmoderrors;

const
  SamplingRate = 44100;

var
  Stream: PFSoundStream;
  Channel : Integer;
  Pos : Integer;
  Freq : Integer;
  CPUUpdate : Integer;

function StreamCallback(Stream: PFSoundStream; Buff: Pointer; Len, Param: Integer): ByteBool; stdcall;
var
  Count: Integer;
  Buffer: PChar;
  CPUUsage : Single;
  Value : Single;
  Delta : Single;

begin
  Freq := 880;
  Buffer := PChar(Buff);
  Count := 0;

  Delta := 1.0 + (1/10000.0 * SchwebGenForm.PercentBar.Position);

  while Count < (Len shr 2) do		{ >>2 = 16bit stereo (4 bytes per sample) }
  begin
    Value := (Sin(2*Pi/SamplingRate*Freq*Pos) + Sin(2*Pi/SamplingRate*Freq*Pos*Delta));
    PSmallInt(Buffer)^ := Trunc(Value * (32767.0/2.0));    { Left channel }
    Inc(Buffer, 2);
    PSmallInt(Buffer)^ := Trunc(Value * (32767.0/2.0));   { Right channel }
    Inc(Buffer, 2);

    Inc(Count);
    Inc(Pos);
  end;

  Inc(CPUUpdate);
  if CPUUpdate > 10 then
  begin
    CPUUsage := FSOUND_GetCPUUsage;
    SchwebGenForm.CPUUsage.Position := Round(CPUUsage*10);
    SchwebGenForm.CPUUsageLabel.Caption := Format('CPU-Last: %f %%', [CPUUsage]);
    CPUUpdate := 0;
  end;

  Result := True;
end;


procedure TSchwebGenForm.PercentBarChange(Sender: TObject);
var
  Percent : Integer;
begin
  Percent := PercentBar.Position;
  PercentLabel.Caption := Format('%f %%', [Percent / 100])
end;

procedure TSchwebGenForm.FormCreate(Sender: TObject);
begin
  Pos := 0;
  CPUUpdate := 10;
  PercentBarChange(Self);

  SoundActive := OpenFMOD;
  if SoundActive then
  begin
    if not FSOUND_Init(SamplingRate, 16, 0) then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der Initialisierung von FMOD ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;

    Stream := FSOUND_Stream_Create(StreamCallback, 2500 * 2 * 2, FSOUND_NORMAL or FSOUND_16BITS or FSOUND_STEREO, SamplingRate, 12345);
    if Stream = nil then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der des Sound-Streams ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;
    Channel := FSOUND_Stream_Play(FSOUND_FREE, Stream);
  end
  else
    LogFile.AddWarning('->FormCreate', 'Soundausgabe nicht mglich!');
end;

procedure TSchwebGenForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if SoundActive then
    FSOUND_Stream_Close(Stream);
  CloseFMOD;
end;

end.
