unit synth4_oberuntertoene;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, fmod_synthutils, engine_rtf, ComCtrls, ExtCtrls,
  spectrum_analyzer, Math, ExtDlgs, synth_functions;

type
  TOberUntertonGenForm = class(TForm)
    FreqBar: TScrollBar;
    FreqLabel: TLabel;
    SpectrumPanel: TPanel;
    CPUUsageLabel: TLabel;
    SpectrumCheck: TCheckBox;
    SpectrumTimer: TTimer;
    PulseLenBar: TScrollBar;
    PulseLenLabel: TLabel;
    WaveModeBar: TScrollBar;
    WaveModeLabel: TLabel;
    GraphBox: TPaintBox;
    SavePictureDialog: TSavePictureDialog;
    TonArt: TGroupBox;
    Obertoene: TRadioButton;
    Untertoene: TRadioButton;
    TonzahlBar: TScrollBar;
    TonZahlLabel: TLabel;
    CPUUsage: TProgressBar;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpectrumTimerTimer(Sender: TObject);
    procedure SpectrumCheckClick(Sender: TObject);
    procedure PulseLenBarChange(Sender: TObject);
    procedure FreqBarChange(Sender: TObject);
    procedure WaveModeBarChange(Sender: TObject);
    procedure GraphBoxPaint(Sender: TObject);
    procedure GraphBoxDblClick(Sender: TObject);
    procedure TonzahlBarChange(Sender: TObject);
    procedure ObertoeneClick(Sender: TObject);
  private
    { Private-Deklarationen }
    SoundActive : Boolean;
    Spectrum : TSpectrum;
    SaveImage : Boolean;
  public
    { Public-Deklarationen }
    function CalcWave(Mode : Integer; x : Single) : Single;
  end;

var
  OberUntertonGenForm: TOberUntertonGenForm;

implementation

{$R *.DFM}

uses
  fmod, fmodtypes, fmoderrors;

const
  SamplingRate = 44100;
  Tonstaerken : Array[1..4] of Integer = (20, 10, 8, 5);

var
  Stream: PFSoundStream;
  Channel : Integer;
  Pos : Integer;
  Freq : Integer;
  CPUUpdate : Integer;

function CalcDivisor(Max : Integer): Single;
var
  i : Integer;
begin
  Result := 0.0;
  for i := 1 to Max do
  begin
    Result := Result + Tonstaerken[i];
  end;
  Result := 1 / Result;
end;

function StreamCallback(Stream: PFSoundStream; Buff: Pointer; Len, Param: Integer): ByteBool; stdcall;
var
  Count: Integer;
  Buffer: PChar;
  CPUUsage : Single;
  Value : Single;
  Mode : Integer;
  i : Integer;
  TonZahl : Integer;
  Obertoene : Boolean;
  Divisor : Single;
  
begin
  Freq := OberUntertonGenForm.FreqBar.Position;
  Buffer := PChar(Buff);
  Count := 0;
  Mode := OberUntertonGenForm.WaveModeBar.Position;
  TonZahl := OberUntertonGenForm.TonzahlBar.Position;
  Obertoene := OberUntertonGenForm.Obertoene.Checked;
  Divisor := CalcDivisor(TonZahl);

  while Count < (Len shr 2) do		{ >>2 = 16bit stereo (4 bytes per sample) }
  begin
    Value := 0.0;
    for i := 1 to TonZahl do
    begin
      if Obertoene then
        Value := Value + Tonstaerken[i] * OberUntertonGenForm.CalcWave(Mode, 1/SamplingRate*Freq*Pos*Power(2,i-1))
      else
        Value := Value + Tonstaerken[i] * OberUntertonGenForm.CalcWave(Mode, 1/SamplingRate*Freq*Pos/Power(2,i-1))
    end;
    Value := Value * Divisor;

    PSmallInt(Buffer)^ := Trunc(Value * (32767.0/2.0));    { Left channel }
    Inc(Buffer, 2);
    PSmallInt(Buffer)^ := Trunc(Value * (32767.0/2.0));   { Right channel }
    Inc(Buffer, 2);

    Inc(Count);
    Inc(Pos);
  end;

  Inc(CPUUpdate);
  if CPUUpdate > 10 then
  begin
    CPUUsage := FSOUND_GetCPUUsage;
    OberUntertonGenForm.CPUUsage.Position := Round(CPUUsage*10);
    OberUntertonGenForm.CPUUsageLabel.Caption := Format('CPU-Last: %f %%', [CPUUsage]);
    CPUUpdate := 0;
  end;

  Result := True;
end;


procedure TOberUntertonGenForm.FormCreate(Sender: TObject);
begin
  TonZahlBar.Max := High(TonStaerken);
  SaveImage := false;
  Pos := 0;
  CPUUpdate := 10;
  FreqBarChange(Self);
  PulseLenBarChange(Self);
  TonZahlBarChange(Self);
  Randomize;

  SoundActive := OpenFMOD;
  if SoundActive then
  begin
    if not FSOUND_Init(SamplingRate, 16, 0) then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der Initialisierung von FMOD ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;

    Stream := FSOUND_Stream_Create(StreamCallback, 2500 * 2 * 2, FSOUND_NORMAL or FSOUND_16BITS or FSOUND_STEREO, SamplingRate, 12345);
    if Stream = nil then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der des Sound-Streams ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;
    Channel := FSOUND_Stream_Play(FSOUND_FREE, Stream);

    Spectrum := TSpectrum.Create(SpectrumPanel);
    Spectrum.Parent := SpectrumPanel;
    Spectrum.Left := 1;
    Spectrum.Top := 1;
    Spectrum.Height := SpectrumPanel.Height -1;
    Spectrum.Visible := true;
    Spectrum.SpectrumEnabled := SpectrumCheck.Checked;
  end
  else
    LogFile.AddWarning('->FormCreate', 'Soundausgabe nicht mglich!');
end;

procedure TOberUntertonGenForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(Spectrum) then
  begin
    Spectrum.Free;
    Spectrum := nil
  end;

  if SoundActive then
    FSOUND_Stream_Close(Stream);
  CloseFMOD;
end;

procedure TOberUntertonGenForm.SpectrumTimerTimer(Sender: TObject);
begin
  if Assigned(Spectrum) and (Spectrum.SpectrumEnabled) then
    Spectrum.RePaint;
end;

procedure TOberUntertonGenForm.SpectrumCheckClick(Sender: TObject);
begin
  Spectrum.SpectrumEnabled := SpectrumCheck.Checked;
end;

function TOberUntertonGenForm.CalcWave(Mode: Integer; x: Single): Single;
begin
  case Mode of
    0: Result := SinWave(x);
    1: Result := DreieckWave(x);
    2: Result := SaegezahnWave(x);
    3: Result := PulseWave(x, 1/1000 * PulseLenBar.Position);
  else
    Result := 0.0;
  end;
end;

procedure TOberUntertonGenForm.PulseLenBarChange(Sender: TObject);
begin
  PulseLenLabel.Caption := Format('Pulse-Lnge: %f %%', [PulseLenBar.Position / 10]);
  GraphBox.Refresh;
end;

procedure TOberUntertonGenForm.FreqBarChange(Sender: TObject);
var
  Freq : Integer;
begin
  Freq := FreqBar.Position;
  if Freq < 1000 then
    FreqLabel.Caption := Format('%d Hz Lambda = %f m', [Freq, 331/Freq])
  else
    FreqLabel.Caption := Format('%f kHz Lambda = %f mm', [Freq/1000, 331/Freq*1000])
end;

procedure TOberUntertonGenForm.WaveModeBarChange(Sender: TObject);
const
  WaveModes : Array[0..3] of String = ('Sinus', 'Dreieck', 'Sgezahn', 'Puls');
begin
  if (WaveModeBar.Position >= 0) and (WaveModeBar.Position <= High(WaveModes)) then
    WaveModeLabel.Caption := 'Wellenart: ' + WaveModes[WaveModeBar.Position]
  else
    WaveModeLabel.Caption := 'Wellenart: undefiniert';
  GraphBox.Refresh;
end;

procedure TOberUntertonGenForm.GraphBoxPaint(Sender: TObject);
var
  Buffer : TBitMap;
  UnitLen, UnitLenTel, Divisor : Single;

  x, max,y, i, TonZahl : Integer;
  ys, Midline : Single;
  Mode : Integer;
  Obertoene : Boolean;
begin
  Buffer := TBitMap.Create;
  try
    Buffer.Width := GraphBox.Width;
    Buffer.Height := GraphBox.Height;

    max := Buffer.Width -2;
    UnitLen := (GraphBox.Height / 2) - 8;
    UnitLenTel := 1 / UnitLen;
    Mode := WaveModeBar.Position;
    Midline := GraphBox.Height / 2;

    with Buffer.Canvas do
    begin
      Pen.Color := clBlack;
      Brush.Color := clWhite;
      Rectangle(0,0, Buffer.Width - 1, Buffer.Height -1);

      MoveTo(0, Buffer.Height div 2);
      LineTo(Buffer.Width, Buffer.Height div 2);

      Pen.Color := clRed;

      TonZahl := TonzahlBar.Position;
      Obertoene := Self.Obertoene.Checked;
      Divisor := CalcDivisor(TonZahl);

      for x := 1 to max do
      begin
        ys := 0.0;
        for i := 1 to TonZahl do
        begin
          if Obertoene then
            ys := ys + Tonstaerken[i] * OberUntertonGenForm.CalcWave(Mode,1/2*(x - 1)*UnitLenTel*Power(2,i-1))
          else
            ys := ys + Tonstaerken[i] * OberUntertonGenForm.CalcWave(Mode, (x - 1)*2.0*UnitLenTel/Power(2,i-1))
        end;
        ys := ys * Divisor;
        ys := -ys * UnitLen + MidLine;

        y := Round(ys);
        if x = 1 then
          MoveTo(x,y)
        else
          LineTo(x,y);
      end;
    end;
    GraphBox.Canvas.Draw(0,0, Buffer);

    if SaveImage then
    begin
      Buffer.SaveToFile(SavePictureDialog.FileName);
      SaveImage := False;
    end;
  finally
    Buffer.Free;
    Buffer := nil;
  end;
end;


procedure TOberUntertonGenForm.GraphBoxDblClick(Sender: TObject);
begin
  SaveImage := SavePictureDialog.Execute;
end;

procedure TOberUntertonGenForm.TonzahlBarChange(Sender: TObject);
const
  Tonart : Array[Boolean] of String = ('Untertne', 'Obertne');
begin
  TonZahlLabel.Caption := Format('%s: %d', [Tonart[Obertoene.Checked], TonZahlBar.Position-1]);
  GraphBoxPaint(Self);
end;

procedure TOberUntertonGenForm.ObertoeneClick(Sender: TObject);
begin
  TonzahlBarChange(Self);
end;

end.

