unit synth5_instrumente;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, fmod_synthutils, engine_rtf, ComCtrls, ExtCtrls,
  spectrum_analyzer, Math, ExtDlgs, synth_functions;

type
  TInstrumentGenForm = class(TForm)
    SpectrumPanel: TPanel;
    CPUUsageLabel: TLabel;
    SpectrumCheck: TCheckBox;
    SpectrumTimer: TTimer;
    SavePictureDialog: TSavePictureDialog;
    InstrumentPanel: TGroupBox;
    Tonlaenge: TScrollBar;
    KeyboardBox: TPaintBox;
    KeyLabel: TLabel;
    CPUUsage: TProgressBar;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpectrumTimerTimer(Sender: TObject);
    procedure SpectrumCheckClick(Sender: TObject);
    procedure InstrumentButtonClick(Sender: TObject);
    procedure KeyboardBoxPaint(Sender: TObject);
    procedure KeyboardBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private-Deklarationen }
    SoundActive : Boolean;
    Spectrum : TSpectrum;
    OctavColorBmp : TBitMap;
    MouseOverKey : Integer;
  public
    { Public-Deklarationen }
  end;

var
  InstrumentGenForm: TInstrumentGenForm;

implementation

{$R *.DFM}

uses
  fmod, fmodtypes, fmoderrors;

type
  //Die einzelnen Instrumente
  TInstrument = (iKlavier, iKlavinettA, iKlavinettB, iEOrgel, iFloete, iFloeteVariant,iGlocke, iHiHat);

  TNote = record //Wichtige Einstellungen zum Abspielen einer Note
            BaseFreq, Duration, x : Single;
            Instrument : TInstrument;
          end;

const
  //Die Namen der spielbaren Instrumente
  InstrumentNamen : Array[TInstrument] of String =
    ('Klavier', 'Klavinett A', 'Klavinett B','Elektro-Orgel', 'Flte/Melodika', 'Fltenvariante','Glocke', 'Hi-Hat');

  SamplingRate = 44100; //Sampling Rate

  KeyWidth = 10;         //Breite der weissen Tasten
  SmallKeyWidthHalf = 3; //Halbe breite der schwarzen Tasten

  ZahlSpieloktaven = 7; //Zahl der auf der Tastatur spielbaren = angezeigten
                        //Oktaven. Die hohen Tasten sind fr obertonreiche
                        //Instrumente, die niedrigen fr untertonreiche+
                        //Instrumente schlecht geeignet.

var
  Stream: PFSoundStream;
  Channel : Integer;
  CPUUpdate : Integer;

  Instrument : TInstrument; //Das durch die Radio-Buttons gewhlte Instrument
  Tasten : Array[0..2] of TNote; //Beherbergt Tasten

const
  //Hllkurveneinstellungen

  (* Klavier *)
  Klavier_Waves_Attack   = 4;
  Klavier_Waves_MaxDecay = 9600;
  Klavier_Waves_End = 500;

  (* Elektro-Orgel *)
  EOrgel_Waves_Attack = 20;
  EOrgel_Waves_Decay = 80;
  EOrgel_Waves_End = 150;

  (* Flte *)
  Floete_Waves_Attack = 50;
  Floete_Waves_End    = 20;

  (* Glocke *)
  Glocke_Waves_Attack   = 15;
  Glocke_Waves_MaxDecay = 40000;
  Glocke_Waves_End = 1800;

function StreamCallback(Stream: PFSoundStream; Buff: Pointer; Len, Param: Integer): ByteBool; stdcall;

  function Obertoene(x, f : Single) : Single;
  {Erzeugt aus Sini ein obertonreiches, normiertes Gesamt.
   x : Zeitlauf; f : Basis-Frequenz des zu erzeugenden Tones}
  begin
    x := x * 2*Pi * f;
    Result :=          20 * Sin(x    );
    Result := Result + 13 * Sin(x * 2);
    Result := Result + 9  * Sin(x * 4);
    Result := Result + 5  * Sin(x * 8);
    Result := 1/(20+13+9+5) * Result;
  end;

  function DreieckObertoene(x, f : Single) : Single;
  { Wie Obertoene, nur mit Dreiecken statt Sini }
  begin
    x := x * f;
    Result :=          20 * DreieckWave(x    );
    Result := Result + 13 * DreieckWave(x * 2);
    Result := Result + 9  * DreieckWave(x * 4);
    Result := Result + 5  * DreieckWave(x * 8);
    Result := 1/(20+13+9+5) * Result;
  end;

  function SaegezahnObertoene(x, f : Single) : Single;
  { Wie Obertoene, nur mit Sgezhnen statt Sini }
  begin
    x := x * f;
    Result :=          20 * SaegezahnWave(x    );
    Result := Result + 13 * SaegezahnWave(x * 2);
    Result := Result + 9  * SaegezahnWave(x * 4);
    Result := Result + 5  * SaegezahnWave(x * 8);
    Result := 1/(20+13+9+5) * Result;
  end;

  function Untertoene(x, f : Single) : Single;
  { Wie Obertoene, nur Untertoene statt Obertoene }
  begin
    x := x * 2*Pi * f;
    Result :=          20 * Sin(x    );
    Result := Result + 13 * Sin(1/2 * x);
    Result := Result + 9  * Sin(1/4 * x);
    Result := Result + 5  * Sin(1/8 * x);
    Result := 1/(20+13+9+5) * Result;
  end;

  function PlayKlavier(Note : TNote): Single;
  { Erzeugt normierte Samples fr Klaviernoten:
    Obertne, Schwebungen }
  begin
    //Sinuskurve
    Result :=          Obertoene(Note.x, Note.BaseFreq);
    Result := Result + Obertoene(Note.x, Note.BaseFreq * 1.0015);
    Result := Result + Obertoene(Note.x, Note.BaseFreq * 1.0025);
    Result := 1/3.0 * Result;
    //Envelope
    Result := Result * ADEnvelopeCutOff(Note.x, Klavier_Waves_Attack/Note.BaseFreq,
                Note.Duration - Klavier_Waves_Attack/Note.BaseFreq -
                Klavier_Waves_End/Note.BaseFreq, Klavier_Waves_MaxDecay / Note.BaseFreq,
                Klavier_Waves_End/Note.BaseFreq, 1.0, 0.0);
  end;

  function PlayKlavinettA(Note : TNote): Single;
  { Erzeugt normierte Samples fr Klavinettnoten:
    Obertne, Schwebungen. Ein Klavinett ist einem Klavier sehr
    hnlich, nur klingt es ein wenig "hrter" }
  begin
    //Sinuskurve
    Result :=          DreieckObertoene(Note.x, Note.BaseFreq);
    Result := Result + DreieckObertoene(Note.x, Note.BaseFreq * 1.0015);
    Result := Result + DreieckObertoene(Note.x, Note.BaseFreq * 1.0025);
    Result := 1/3.0 * Result;
    //Envelope
    Result := Result * ADEnvelopeCutOff(Note.x, Klavier_Waves_Attack/Note.BaseFreq,
                Note.Duration - Klavier_Waves_Attack/Note.BaseFreq -
                Klavier_Waves_End/Note.BaseFreq, Klavier_Waves_MaxDecay / Note.BaseFreq,
                Klavier_Waves_End/Note.BaseFreq, 1.0, 0.0);
  end;

  function PlayKlavinettB(Note : TNote): Single;
  { Erzeugt normierte Samples fr Klavinettnoten:
    Obertne, Schwebungen. Ein Klavinett ist einem Klavier sehr
    hnlich, nur klingt es ein wenig "hrter" }
  begin
    //Sinuskurve
    Result :=          SaegezahnObertoene(Note.x, Note.BaseFreq);
    Result := Result + SaegezahnObertoene(Note.x, Note.BaseFreq * 1.0015);
    Result := Result + SaegezahnObertoene(Note.x, Note.BaseFreq * 1.0025);
    Result := 1/3.0 * Result;
    //Envelope
    Result := Result * ADEnvelopeCutOff(Note.x, Klavier_Waves_Attack/Note.BaseFreq,
                Note.Duration - Klavier_Waves_Attack/Note.BaseFreq -
                Klavier_Waves_End/Note.BaseFreq, Klavier_Waves_MaxDecay / Note.BaseFreq,
                Klavier_Waves_End/Note.BaseFreq, 1.0, 0.0);
  end;

  function PlayEOrgel(Note : TNote): Single;
  { Erzeugt normierte Samples fr Elektro-Orgeln, die
    hufig bei Gospeln zum Einsatz kommen. Die schwebung ist
    sehr ungewhnlich und klingt sehr charakteristisch. Die
    Hllkurve verluft ein wenig anders, als beim Klavier. }
  begin
    //Sinuskurve
    Result :=          Obertoene(Note.x, Note.BaseFreq);
    Result := Result + Obertoene(Note.x, Note.BaseFreq * 1.0175);
    Result := Result + Obertoene(Note.x, Note.BaseFreq * 1.025);
    Result := 1/3.0 * Result;
    //Envelope
    Result := Result * ADSREnvelope(Note.x, EOrgel_Waves_Attack/Note.BaseFreq,
                         EOrgel_Waves_Decay/Note.BaseFreq,
                         Note.Duration - EOrgel_Waves_Attack/Note.BaseFreq -
                           EOrgel_Waves_Decay/Note.BaseFreq - EOrgel_Waves_End/Note.BaseFreq,
                         EOrgel_Waves_End/Note.BaseFreq, 1.0, 0.7);
  end;

  function PlayFloete(Note : TNote): Single;
  { Erzeugt normierte Samples fr Floeten/Melodikas.
    Das ganze luft ber sehr einfache Dreiecke. Eine Reihe
    von Verbesserungen wren sicher mglich. }
  begin
    Result := DreieckWave(Note.x * Note.BaseFreq)*Sin(Note.x * 2*Pi*Note.BaseFreq*2);
    //Envelope
    Result := Result * ASREnvelope(Note.x, Floete_Waves_Attack/Note.BaseFreq,
                         Note.Duration - Floete_Waves_Attack/Note.BaseFreq -
                           Floete_Waves_End/Note.BaseFreq,
                           Floete_Waves_End/Note.BaseFreq, 1.0);
  end;

  function PlayFloeteVariante(Note : TNote): Single;
  { Erzeugt normierte Samples fr Floeten/Melodikas.
    Das ganze luft ber sehr einfache Dreiecke. Eine Reihe
    von Verbesserungen wren sicher mglich. }
  begin
    Result := SaegezahnWave(Note.x * Note.BaseFreq)*Sin(Note.x * 2*Pi*Note.BaseFreq*2);
    //Envelope
    Result := Result * ASREnvelope(Note.x, Floete_Waves_Attack/Note.BaseFreq,
                         Note.Duration - Floete_Waves_Attack/Note.BaseFreq -
                           Floete_Waves_End/Note.BaseFreq,
                           Floete_Waves_End/Note.BaseFreq, 1.0);
  end;

  function PlayGlocke(Note : TNote): Single;
  { Erzeugt normierte Samples fr Glocken: Untertne mit
    Schwebungen }
  begin
    Result :=          Untertoene(Note.x, Note.BaseFreq);
    Result := Result + Untertoene(Note.x, Note.BaseFreq * 1.005);
    Result := Result + Untertoene(Note.x, Note.BaseFreq * 1.0075);
    Result := 1/3.0 * Result;

    //Envelope
    Result := Result * ADEnvelopeCutOff(Note.x, Glocke_Waves_Attack/Note.BaseFreq,
                Note.Duration - Glocke_Waves_Attack/Note.BaseFreq -
                Glocke_Waves_End/Note.BaseFreq, Glocke_Waves_MaxDecay / Note.BaseFreq,
                Glocke_Waves_End/Note.BaseFreq, 1.0, 0.0);
  end;

  function PlayHiHat(Note : TNote): Single;
  { Erzeugt normierte Samples fr Hi-Hats. Das gengt im normalfall
    und fllt, wenn das Hi-Hat nich alleine auftritt eigentlich
    kaum auf. }
  begin
    if Note.x < 0.03 then
      Result := 1 / 3.0 * Random
    else
      Result := 0
  end;

var
  Buffer: PChar;
  CPUUsage : Single;
  Value : Single;
  Key : Integer;
  Count: Integer;

begin
  Buffer := PChar(Buff);
  Count := 0;

  while Count < (Len shr 2) do		{ >>2 = 16bit stereo (4 bytes per sample) }
  begin
    Value := 0;
    //Noten spielen, die noch was von sich geben
    for Key := 0 to High(Tasten) do
    begin
      if (Tasten[Key].x <= Tasten[Key].Duration) and (Tasten[Key].Duration > 0.0) then
      begin

        case Tasten[Key].Instrument of
          iKlavier       : Value := Value + PlayKlavier(Tasten[Key]);
          iKlavinettA    : Value := Value + PlayKlavinettA(Tasten[Key]);
          iKlavinettB    : Value := Value + PlayKlavinettB(Tasten[Key]);
          iEOrgel        : Value := Value + PlayEOrgel(Tasten[Key]);
          iFloete        : Value := Value + PlayFloete(Tasten[Key]);
          iFloeteVariant : Value := Value + PlayFloeteVariante(Tasten[Key]);
          iGlocke        : Value := Value + PlayGlocke(Tasten[Key]);
          iHiHat         : Value := Value + PlayHiHat(Tasten[Key]);
        else

        end;

        Tasten[Key].x := Tasten[Key].x + (1 / SamplingRate);
      end;
    end;
    Value := 1/(High(Tasten) + 1) * Value;
      //berlufe wegen zu vielen, gedrckten Tasten sind absoluter Mist

    PSmallInt(Buffer)^ := Trunc(Value * (32767.0));    { Left channel }
    Inc(Buffer, 2);
    PSmallInt(Buffer)^ := Trunc(Value * (32767.0));   { Right channel }
    Inc(Buffer, 2);

    Inc(Count);
  end;

  Inc(CPUUpdate);
  if CPUUpdate > 10 then
  begin
    CPUUsage := FSOUND_GetCPUUsage;
    InstrumentGenForm.CPUUsage.Position := Round(CPUUsage*10);
    InstrumentGenForm.CPUUsageLabel.Caption := Format('CPU-Last: %f %%', [CPUUsage]);
    CPUUpdate := 0;
  end;

  Result := True;
end;


procedure TInstrumentGenForm.FormCreate(Sender: TObject);
var
  i : TInstrument;
  RadioButton : TRadioButton;
  Freq : Single;
  Key : Integer;

begin
  CPUUpdate := 10;

  FillChar(Tasten, SizeOf(Tasten), 0);

  for i := Low(TInstrument) to High(TInstrument) do
  begin
    RadioButton := TRadioButton.Create(InstrumentPanel);
    RadioButton.Parent := InstrumentPanel;
    RadioButton.Left := 12;
    RadioButton.Top := 16 + Ord(i) * 16;
    RadioButton.Tag := Ord(i);
    RadioButton.Checked := Ord(i) = 0;
    RadioButton.OnClick := InstrumentButtonClick;
    RadioButton.Caption := InstrumentNamen[i];
  end;
  Instrument := iKlavier;


  SoundActive := OpenFMOD;
  if SoundActive then
  begin
    if not FSOUND_Init(SamplingRate, 16, 0) then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der Initialisierung von FMOD ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;

    Stream := FSOUND_Stream_Create(StreamCallback, 2500 * 2 * 2, FSOUND_NORMAL or FSOUND_16BITS or FSOUND_STEREO, SamplingRate, 12345);
    if Stream = nil then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der des Sound-Streams ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;
    Channel := FSOUND_Stream_Play(FSOUND_FREE, Stream);

    Spectrum := TSpectrum.Create(SpectrumPanel);
    Spectrum.Parent := SpectrumPanel;
    Spectrum.Left := 1;
    Spectrum.Top := 1;
    Spectrum.Height := SpectrumPanel.Height -1;
    Spectrum.Visible := true;
    Spectrum.SpectrumEnabled := SpectrumCheck.Checked;
  end
  else
    LogFile.AddWarning('->FormCreate', 'Soundausgabe nicht mglich!');
end;

procedure TInstrumentGenForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(OctavColorBmp) then
  begin
    OctavColorBmp.Free;
    OctavColorBmp := nil;
  end;

  if Assigned(Spectrum) then
  begin
    Spectrum.Free;
    Spectrum := nil
  end;

  if SoundActive then
    FSOUND_Stream_Close(Stream);
  CloseFMOD;
end;

procedure TInstrumentGenForm.SpectrumTimerTimer(Sender: TObject);
begin
  if Assigned(Spectrum) and (Spectrum.SpectrumEnabled) then
    Spectrum.Invalidate;
end;

procedure TInstrumentGenForm.SpectrumCheckClick(Sender: TObject);
begin
  Spectrum.SpectrumEnabled := SpectrumCheck.Checked;
end;

procedure TInstrumentGenForm.InstrumentButtonClick(Sender: TObject);
begin
  if Sender is TRadioButton then
  begin
    Instrument := TInstrument((Sender as TRadioButton).Tag)
  end
end;

procedure TInstrumentGenForm.KeyboardBoxPaint(Sender: TObject);
var
  KeyHeight : Integer;

  procedure PaintOctaveKeys(Bitmap : TCanvas; Colored : Boolean; TransX : Integer);
  const
    VisibleColoredKeys : Array[0..11] of Array[0..2] of Byte =
      ((255,0,0), (0,255,0), (255, 255, 0), (0, 0, 255),
       (255,0,255), (0, 255, 255), (128, 0,0), (0, 128, 0),
       (128, 128,0), (0,0,128), (128,0,128), (0, 128, 128));
  var
    i : Integer;
    c : Integer;
  begin
    //Weisse Tasten
    for i := 0 to 6 do
    begin
      if Colored then
      begin
        c := i * 2;
        if c > 5 then
          Dec(c);
        {Bitmap.Pen.Color := RGB(VisibleColoredKeys[c][0], VisibleColoredKeys[c][1],
                         VisibleColoredKeys[c][2]);}
        Bitmap.Pen.Color := c;
        Bitmap.Brush.Color := Bitmap.Pen.Color;
      end
      else
      begin
        Bitmap.Pen.Color := clBlack;
        Bitmap.Brush.COlor := clWhite;
        Bitmap.Pen.Width := 1;
      end;
      Bitmap.Rectangle(TransX + i * KeyWidth, 0, TransX + (i+1)*KeyWidth+1, KeyHeight -1);
    end;

    for i := 1 to 6 do
    begin
      if i = 3 then
        Continue;

      if Colored then
      begin
        c := (i-1) * 2+1;
        if c > 5 then
          Dec(c);
        {Bitmap.Pen.Color := RGB(VisibleColoredKeys[c][0], VisibleColoredKeys[c][1],
                         VisibleColoredKeys[c][2]);}
        Bitmap.Pen.Color := c;
        Bitmap.Brush.Color := Bitmap.Pen.Color;
      end
      else
      begin
        Bitmap.Pen.Color := clBlack;
        Bitmap.Brush.COlor := clBlack;
        Bitmap.Pen.Width := 1;
      end;
      Bitmap.Rectangle(TransX + i * KeyWidth - SmallKeyWidthHalf, 0, TransX + i*KeyWidth+1+SmallKeyWidthHalf, 2* KeyHeight div 3);
    end;
  end;

var
  i : Integer;
begin
  KeyHeight := KeyboardBox.Height;
  if not Assigned(OctavColorBmp) then
  begin
    OctavColorBmp := TBitMap.Create;
    OctavColorBmp.Height := KeyHeight;
    OctavColorBmp.Width := KeyWidth * 7+1;
    PaintOctaveKeys(OctavColorBmp.Canvas, True, 0);
  end;
  for i := 0 to ZahlSpieloktaven -1  do
  begin
    PaintOctaveKeys(KeyBoardBox.Canvas, False, i * 7 * KeyWidth);
  end;
end;

procedure TInstrumentGenForm.KeyboardBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Oktave : Integer;
  Ton, i : Integer;
  SustainTime : Single;
begin
  Oktave := X div (7 * KeyWidth);
  Ton := X - (7*KeyWidth * Oktave);
  if Assigned(OctavColorBmp) then
  begin
    Ton := OctavColorBmp.Canvas.Pixels[Ton, Y];
  end;
  KeyLabel.Caption := Format('Key-Info: O: %d T: %d', [Oktave, Ton]);
  for i := 0 to High(Tasten) do
  begin
    if (Tasten[i].x > Tasten[i].Duration) or (Tasten[i].Duration = 0) then
    begin
      Tasten[i].x := 0.0;
      Tasten[i].BaseFreq := 65.25 * Power(1.059463094, Ton) * Power(2, Oktave);
      Tasten[i].Instrument := Instrument;
      Tasten[i].Duration := Tonlaenge.Position / 1000;

      case Tasten[i].Instrument of
        iKlavier, iKlavinettA, iKlavinettB : begin
                     SustainTime := Tasten[i].Duration
                       - Klavier_Waves_Attack/Tasten[i].BaseFreq
                       - Klavier_Waves_End/Tasten[i].BaseFreq;
                     //Note bei Bedarf verlngern
                     if SustainTime < 0 then
                       Tasten[i].Duration := Tasten[i].Duration - SustainTime;
                   end;
        iEOrgel  : begin
                     SustainTime := Tasten[i].Duration
                       - EOrgel_Waves_Attack/Tasten[i].BaseFreq
                       - EOrgel_Waves_Decay/Tasten[i].BaseFreq
                       - EOrgel_Waves_End/Tasten[i].BaseFreq;
                     //Note bei Bedarf verlngern
                     if SustainTime < 0 then
                       Tasten[i].Duration := Tasten[i].Duration - SustainTime;
                   end;
        iFloete, iFloeteVariant  : begin
                     SustainTime := Tasten[i].Duration
                       - Floete_Waves_Attack/Tasten[i].BaseFreq
                       - Floete_Waves_End/Tasten[i].BaseFreq;
                     //Note bei Bedarf verlngern
                     if SustainTime < 0 then
                       Tasten[i].Duration := Tasten[i].Duration - SustainTime;
                   end;
        iGlocke  : begin
                     SustainTime := Tasten[i].Duration
                       - Glocke_Waves_Attack/Tasten[i].BaseFreq
                       - Glocke_Waves_End/Tasten[i].BaseFreq;
                     //Note bei Bedarf verlngern
                     if SustainTime < 0 then
                       Tasten[i].Duration := Tasten[i].Duration - SustainTime;
                   end;
        iHiHat   : begin
                     Tasten[i].Duration := 0.03;
                   end;
      end;
      Break;
    end;
  end;
end;

end.

