unit synth_functions;

interface

function Sgn(x : Single) : Single;

function DreieckWave(x: Single): Single;
function PulseWave(x, Breite: Single): Single;
function SaegezahnWave(x: Single): Single;
function SinWave(x: Single): Single;

function ADSREnvelope(t, a, d, s, r, maxlevel, holdlevel : Single): Single;
function ADEnvelopeCutOff(t, a, d, dmax, r, maxlevel, holdlevel : Single): Single;
function ASREnvelope(t, a, s, r, maxlevel : Single): Single;

implementation

uses
  Math;

const
  e = 2.71828182846;

function Sgn(x : Single) : Single;
begin
  if x < 0 then
    Result := -1.0
  else
    Result := +1.0;
end;

function DreieckWave(x: Single): Single;
begin
  Result := Abs((frac(x) - 0.5) * 2*2)-1;
end;

function PulseWave(x, Breite: Single): Single;
begin
  Result := Sgn(frac(x) - Breite);
end;

function SaegezahnWave(x: Single): Single;
begin
  Result := (-1 + 2 * frac(x));
end;

function SinWave(x: Single): Single;
begin
  Result := Sin(2*Pi*x);
end;

function ADSREnvelope(t, a, d, s, r, maxlevel, holdlevel : Single): Single;
const
  Start = -4.65;
  Intervallaenge = 1.0 - Start;

var
  Pos, x, delta : Single;
begin
  if t < 0 then
  begin
    Result := 0;
    Exit;
  end;

  //Attack
  Pos := a;
  if t < Pos then
  begin
    x := 1.0 - t/a; //x zwischen 0 und 1
    x := x * Intervallaenge + Start; //x auf das Intervall mappen
    Result := (1.0 - 1/e*(Power(e, x)))*maxlevel;
    Exit;
  end;

  //Decay
  Pos := Pos + d;
  if t <= Pos then
  begin
    delta := maxlevel - holdlevel;
    if delta <= 0.0 then
    begin
      Result := holdlevel;
      Exit
    end;
    x := 1.0 - (t - Pos + d)/d;
    x := x * Intervallaenge + Start;
    Result := Maxlevel - (1.0 - 1/e*(Power(e, x)))*delta;
    Exit;
  end;

  //Sustain
  Pos := Pos + s;
  if t <= Pos then
  begin
    Result := holdlevel;
    Exit
  end;

  //Release
  Pos := Pos + r;
  if t <= Pos then
  begin
    x := 1.0 - (t - Pos + r)/r;
    x := x * Intervallaenge + Start;
    Result := holdlevel - holdlevel * (1.0 - 1/e*(Power(e, x)));
    Exit;
  end;

  Result := 0.0;
end;

function ADEnvelopeCutOff(t, a, d, dmax, r, maxlevel, holdlevel : Single): Single;
const
  Start = -4.65;
  Intervallaenge = 1.0 - Start;

var
  Pos, x, delta : Single;
begin
  if t < 0 then
  begin
    Result := 0;
    Exit;
  end;
  if d > dmax then
    d := dmax;

  //Attack
  Pos := a;
  if t < Pos then
  begin
    x := 1.0 - t/a; //x zwischen 0 und 1
    x := x * Intervallaenge + Start; //x auf das Intervall mappen
    Result := (1.0 - 1/e*(Power(e, x)))*maxlevel;
    Exit;
  end;

  //Decay
  Pos := Pos + d;
  if t <= Pos then
  begin
    delta := maxlevel - holdlevel;
    if delta <= 0.0 then
    begin
      Result := holdlevel;
      Exit
    end;
    x := 1.0 - (t - Pos + d)/dmax;
    x := x * Intervallaenge + Start;
    Result := Maxlevel - (1.0 - 1/e*(Power(e, x)))*delta;
    Exit;
  end;

  begin
    delta := maxlevel - holdlevel;
    if delta <= 0.0 then
    begin
      Result := holdlevel;
      Exit
    end;
    x := 1.0 - d/dmax;
    x := x * Intervallaenge + Start;
    holdlevel := Maxlevel - (1.0 - 1/e*(Power(e, x)))*delta;
  end;

  //Release
  Pos := Pos + r;
  if t <= Pos then
  begin
    x := 1.0 - (t - Pos + r)/r;
    x := x * Intervallaenge + Start;
    Result := holdlevel - holdlevel * (1.0 - 1/e*(Power(e, x)));
    Exit;
  end;

  Result := 0.0;
end;

function ASREnvelope(t, a, s, r, maxlevel : Single): Single;
const
  Start = -4.65;
  Intervallaenge = 1.0 - Start;

var
  Pos, x, delta : Single;
begin
  if t < 0 then
  begin
    Result := 0;
    Exit;
  end;

  //Attack
  Pos := a;
  if t < Pos then
  begin
    x := 1.0 - t/a; //x zwischen 0 und 1
    x := x * Intervallaenge + Start; //x auf das Intervall mappen
    Result := (1.0 - 1/e*(Power(e, x)))*maxlevel;
    Exit;
  end;

  //Sustain
  Pos := Pos + s;
  if t <= Pos then
  begin
    Result := maxlevel;
    Exit
  end;

  //Release
  Pos := Pos + r;
  if t <= Pos then
  begin
    x := 1.0 - (t - Pos + r)/r;
    x := x * Intervallaenge + Start;
    Result := maxlevel - maxlevel * (1.0 - 1/e*(Power(e, x)));
    Exit;
  end;

  Result := 0.0;
end;


end.
